package com.zanxiang.game.back.serve.rpc.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.game.back.base.pojo.dto.*;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.base.rpc.ITencentAppApiBackRpc;
import com.zanxiang.game.back.base.rpc.ITencentAppBackRpc;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.*;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.encryption.Md5Util;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Objects;

@Slf4j
@DubboService
public class TencentAppApiBackRpcImpl implements ITencentAppApiBackRpc {
    @Autowired
    private IGameTencentAppApiUserService gameTencentAppApiUserService;
    @Autowired
    private IGameTencentAppApiRoleRegisterService gameTencentAppApiRoleRegisterService;
    @Autowired
    private IGameTencentAppApiOrderService gameTencentAppApiOrderService;
    @Autowired
    private IGameTencentAppCallbackService gameTencentAppCallbackService;

    @Override
    public ResultVO<Boolean> backUser(TencentUserAppApiRpcDTO dto) {
        log.error("腾讯 APP-API 用户回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameTencentAppApiUser gameTencentUser = GameTencentAppApiUser.builder()
                .userId(dto.getUserId())
                .adAccountId(dto.getAdAccountId())
                .gameId(dto.getGameId())
                .channel(dto.getChannel())
                .subscribeTime(dto.getSubscribeTime())
                .registerTime(dto.getRegisterTime())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .createTime(LocalDateTime.now())
                .build();
        gameTencentAppApiUserService.save(gameTencentUser);
        return ResultVO.ok(true);
        // 激活现在默认不回传了，等创角的时候一起回传
        // return ResultVO.ok(gameTencentAppApiUserService.userBack(gameTencentUser, false));
    }

    @Override
    public ResultVO<Boolean> backRoleRegister(TencentRoleRegisterAppApiRpcDTO dto) {
        log.error("腾讯 APP创角回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameTencentAppApiRoleRegister roleRegister = GameTencentAppApiRoleRegister.builder()
                .userId(dto.getUserId())
                .backPolicyId(dto.getBackPolicyId())
                .channel(dto.getChannel())
                .gameId(dto.getGameId())
                .adAccountId(dto.getAdAccountId())
                .registerTime(dto.getRegisterTime())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .createTime(LocalDateTime.now())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .build();
        gameTencentAppApiRoleRegisterService.save(roleRegister);
        return ResultVO.ok(gameTencentAppApiRoleRegisterService.roleRegisterBack(roleRegister));
    }

    @Override
    public ResultVO<Boolean> backOrder(TencentOrderAppApiRpcDTO dto) {
        if (Objects.equals(OrderStatusEnum.SUCCESS_PAY.getValue(), dto.getOrderStatus())) {
            log.error("腾讯 APP订单回传收到：{}。", JsonUtil.toString(dto));
        } else {
            log.error("腾讯H5订单回传收到：{}。订单未支付，直接过滤", JsonUtil.toString(dto));
            return ResultVO.ok(true);
        }
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid())) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        boolean isFirstOrder = gameTencentAppApiOrderService.getOne(new LambdaQueryWrapper<GameTencentAppApiOrder>()
                .select(GameTencentAppApiOrder::getOrderId)
                .eq(GameTencentAppApiOrder::getGameId, dto.getGameId())
                .eq(GameTencentAppApiOrder::getAdAccountId, dto.getAdAccountId())
                .eq(GameTencentAppApiOrder::getUserId, dto.getUserId())
                .eq(GameTencentAppApiOrder::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                .last("limit 1")
        ) == null;
        //订单保存
        GameTencentAppApiOrder gameTencentOrder = GameTencentAppApiOrder.builder()
                .userId(dto.getUserId())
                .adAccountId(dto.getAdAccountId())
                .gameId(dto.getGameId())
                .orderId(dto.getOrderId())
                .channel(dto.getChannel())
                .rechargeMoney(dto.getRechargeMoney())
                .rechargeTime(dto.getRechargeTime())
                .subscribeTime(dto.getSubscribeTime())
                .registerTime(dto.getRegisterTime())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .orderStatus(dto.getOrderStatus())
                .payTime(dto.getPayTime())
                .createTime(LocalDateTime.now())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .backPolicyId(dto.getBackPolicyId())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .isFirstOrder(isFirstOrder)
                .build();
        gameTencentAppApiOrderService.save(gameTencentOrder);
        return ResultVO.ok(gameTencentAppApiOrderService.orderBack(gameTencentOrder));
    }

    @Override
    public ResultVO<String> queryUserAgentFromCallback(TencentAppApiUserAgentQueryRpcDTO dto) {
        GameTencentAppCallback callback = gameTencentAppCallbackService.getUserCallback(dto.getGameId(), dto.getImei(), dto.getOaid(), dto.getAndroidId(), dto.getIdfa(), dto.getCaid());
        return ResultVO.ok(callback == null ? null : callback.getAgentKey());
    }
}
