package com.zanxiang.game.back.serve.rpc.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.game.back.base.pojo.dto.*;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.base.pojo.vo.OrderBackQueryRpcVO;
import com.zanxiang.game.back.base.rpc.ITtAppBackRpc;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.*;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.rpc.IAgentRpc;
import com.zanxiang.module.redis.service.IDistributedLockComponent;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

@Slf4j
@DubboService
public class TtAppBackRpcImpl implements ITtAppBackRpc {

    @Autowired
    private IGameBackPolicyService gameBackPolicyService;
    @DubboReference(providedBy = ServerInfo.SERVER_DUBBO_NAME)
    private IAgentRpc agentRpc;
    @Autowired
    private IGameOceanengineAppUserLogService gameOceanengineAppUserLogService;
    @Autowired
    private IGameOceanengineAppRoleRegisterLogService gameOceanengineAppRoleRegisterLogService;
    @Autowired
    private IGameOceanengineAppOrderLogService gameOceanengineAppOrderLogService;
    @Autowired
    private IDistributedLockComponent distributedLockComponent;
    @Autowired
    private IGameOceanengineAppCallbackService gameOceanengineAppCallbackService;

    @Override
    public ResultVO<Boolean> userActiveReport(TtUserActiveAppRpcDTO dto) {
        log.error("头条 APP用户回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getUserId()) || (StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid()))) {
            return ResultVO.fail("找不到用户唯一标识");
        }

        GameOceanengineAppUserLog userLog = GameOceanengineAppUserLog.builder()
                .userId(dto.getUserId())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .gameId(dto.getGameId())
                .accountId(dto.getAccountReport().getAccountId())
                .agentKey(dto.getAgentKey())
                .activeTime(dto.getActiveTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .createTime(LocalDateTime.now())
                .build();
        gameOceanengineAppUserLogService.save(userLog);
        return ResultVO.ok(gameOceanengineAppUserLogService.callback(userLog, Boolean.FALSE));
    }

    @Override
    public ResultVO<Boolean> orderReport(TtOrderAppRpcDTO dto) {
        if (Objects.equals(OrderStatusEnum.SUCCESS_PAY.getValue(), dto.getOrderStatus())) {
            log.error("头条 APP订单回传收到：{}。", JsonUtil.toString(dto));
        } else {
            log.error("头条 APP订单回传收到：{}。订单未支付，直接过滤", JsonUtil.toString(dto));
            return ResultVO.ok(true);
        }
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getUserId()) || (StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid()))) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        // 此处加锁防止 cp 方调用过快，导致查询回传结果时判断逻辑没走完
        String lockKey = com.zanxiang.game.back.base.ServerInfo.SERVER_NAME + ":TtAppOrderBack:" + dto.getGameId() + ":" + dto.getOrderId();
        distributedLockComponent.doLock(lockKey, 0L, 60L, TimeUnit.SECONDS);
        try {
            GameOceanengineAppOrderLog orderLog = GameOceanengineAppOrderLog.builder()
                    .userId(dto.getUserId())
                    .mac(dto.getMac())
                    .imei(dto.getImei())
                    .oaid(dto.getOaid())
                    .androidId(dto.getAndroidId())
                    .idfa(dto.getIdfa())
                    .caid(dto.getCaid())
                    .gameId(dto.getGameId())
                    .backPolicyId(dto.getBackPolicyId())
                    .accountId(dto.getAccountReport().getAccountId())
                    .agentKey(dto.getAgentKey())
                    .regTime(dto.getRegTime())
                    .orderNo(dto.getOrderId())
                    .amount(dto.getRechargeMoney())
                    .orderStatus(dto.getOrderStatus())
                    .createTime(dto.getCreateTime())
                    .payTime(dto.getPayTime())
                    .backStatus(BackStatusEnum.NO.getBackStatus())
                    .roleId(dto.getRoleId())
                    .roleName(dto.getRoleName())
                    .build();
            boolean isFirstOrder = gameOceanengineAppOrderLogService.getOne(new LambdaQueryWrapper<GameOceanengineAppOrderLog>()
                    .eq(GameOceanengineAppOrderLog::getGameId, orderLog.getGameId())
                    .eq(GameOceanengineAppOrderLog::getUserId, orderLog.getUserId())
                    .eq(GameOceanengineAppOrderLog::getAccountId, orderLog.getAccountId())
                    .eq(GameOceanengineAppOrderLog::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                    .last("limit 1")
            ) == null;
            orderLog.setIsFirstOrder(isFirstOrder);
            gameOceanengineAppOrderLogService.save(orderLog);
            return ResultVO.ok(gameOceanengineAppOrderLogService.callback(orderLog));
        } finally {
            distributedLockComponent.unlock(lockKey);
        }
    }

    @Override
    public ResultVO<Boolean> roleRegisterReport(TtRoleRegisterAppRpcDTO dto) {
        log.error("头条 APP创角回传收到：{}", JsonUtil.toString(dto));
        if (StringUtils.isBlank(dto.getMac())) {
            dto.setMac("");
        }
        if(StringUtils.isBlank(dto.getImei())) {
            dto.setImei("");
        }
        if(StringUtils.isBlank(dto.getOaid())) {
            dto.setOaid("");
        }
        if(StringUtils.isBlank(dto.getAndroidId())) {
            dto.setAndroidId("");
        }
        if(StringUtils.isBlank(dto.getIdfa())) {
            dto.setIdfa("");
        }
        if(StringUtils.isBlank(dto.getCaid())) {
            dto.setCaid("");
        }
        if(StringUtils.isBlank(dto.getUserId()) || (StringUtils.isBlank(dto.getImei())
                && StringUtils.isBlank(dto.getOaid())
                && StringUtils.isBlank(dto.getAndroidId())
                && StringUtils.isBlank(dto.getIdfa())
                && StringUtils.isBlank(dto.getCaid()))) {
            return ResultVO.fail("找不到用户唯一标识");
        }
        GameOceanengineAppRoleRegisterLog roleRegisterLog = GameOceanengineAppRoleRegisterLog.builder()
                .userId(dto.getUserId())
                .mac(dto.getMac())
                .imei(dto.getImei())
                .oaid(dto.getOaid())
                .androidId(dto.getAndroidId())
                .idfa(dto.getIdfa())
                .caid(dto.getCaid())
                .gameId(dto.getGameId())
                .accountId(dto.getAccountReport().getAccountId())
                .agentKey(dto.getAgentKey())
                .roleId(dto.getRoleId())
                .roleName(dto.getRoleName())
                .registerTime(dto.getRegisterTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .createTime(LocalDateTime.now())
                .build();
        gameOceanengineAppRoleRegisterLogService.save(roleRegisterLog);
        return ResultVO.ok(gameOceanengineAppRoleRegisterLogService.callback(roleRegisterLog));
    }

    @Override
    public ResultVO<Boolean> userBackQuery(TtAppUserBackQueryRpcDTO dto) {
        log.error("头条APP用户回传结果查询：{}", JsonUtil.toString(dto));
        return ResultVO.ok(Boolean.TRUE);
    }

    @Override
    public ResultVO<OrderBackQueryRpcVO> orderBackQuery(TtAppOrderBackQueryRpcDTO dto) {
        log.error("头条APP订单回传结果查询：{}", JsonUtil.toString(dto));

        // 加锁最大等待 60s
        String lockKey = com.zanxiang.game.back.base.ServerInfo.SERVER_NAME + ":TtAppOrderBack:" + dto.getGameId() + ":" + dto.getOrderId();
        distributedLockComponent.doLock(lockKey, 60, 1L, TimeUnit.SECONDS);

        GameOceanengineAppOrderLog orderLog = gameOceanengineAppOrderLogService.getOne(new LambdaQueryWrapper<GameOceanengineAppOrderLog>()
                .eq(GameOceanengineAppOrderLog::getOrderNo, dto.getOrderId())
                .eq(GameOceanengineAppOrderLog::getGameId, dto.getGameId())
                .eq(GameOceanengineAppOrderLog::getAgentKey, dto.getAgentKey())
                .eq(GameOceanengineAppOrderLog::getUserId, dto.getUserId())
        );
        if  (orderLog == null) {
            return ResultVO.ok(OrderBackQueryRpcVO.builder()
                            .doBack(Boolean.FALSE)
                            .backMsg("回传异常，找不到订单：" + dto.getOrderId())
                    .build());
        }
        return ResultVO.ok(OrderBackQueryRpcVO.builder()
                        .doBack(BackStatusEnum.getByValue(orderLog.getBackStatus()) ==  BackStatusEnum.SUCCESS)
                        .backMoney(orderLog.getBackMoney())
                        .backMsg(orderLog.getBackMsg())
                .build());
    }

    @Override
    public ResultVO<Boolean> roleRegisterBackQuery(TtAppRoleRegisterBackQueryRpcDTO dto) {
        log.error("头条APP创角回传结果查询：{}", JsonUtil.toString(dto));
        return ResultVO.ok(Boolean.TRUE);
    }

    @Override
    public ResultVO<String> queryUserAgentFromCallback(TtAppUserAgentQueryRpcDTO dto) {
        GameOceanengineAppCallback callback = gameOceanengineAppCallbackService.getUserCallback(dto.getGameId(), dto.getImei(), dto.getOaid(), dto.getAndroidId(), dto.getIdfa(), dto.getCaid());
        return ResultVO.ok(callback == null ? null : callback.getAgentKey());
    }
}
