package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentAppApiUserMapper;
import com.zanxiang.game.back.serve.pojo.TencentAppReport;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentAppApiBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentAppApiUser;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentAppCallback;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameTencentAppApiBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentAppApiUserService;
import com.zanxiang.game.back.serve.service.IGameTencentAppCallbackService;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.URIUtil;
import com.zanxiang.module.util.encryption.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;

@Slf4j
@Service
public class GameTencentAppApiUserServiceImpl extends ServiceImpl<GameTencentAppApiUserMapper, GameTencentAppApiUser>
implements IGameTencentAppApiUserService {
    @Autowired
    private IGameTencentAppApiBackLogService gameTencentAppApiBackLogService;
    @Autowired
    private IGameTencentAppCallbackService gameTencentAppCallbackService;
    @Autowired
    private RestTemplate restTemplate;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean userBack(GameTencentAppApiUser userLog, boolean mustBack) {
        BackStatusEnum backStatus = doCallback(userLog, mustBack);
        return update(new LambdaUpdateWrapper<GameTencentAppApiUser>()
                .set(GameTencentAppApiUser::getIsBack, backStatus.getBackStatus())
                .eq(GameTencentAppApiUser::getId, userLog.getId())
        );
    }

    @Override
    public boolean tencentUserReport(List<Long> ids) {
        listByIds(ids).forEach(userLog -> {
            BackStatusEnum backStatus = doCallback(userLog, true);
            update(new LambdaUpdateWrapper<GameTencentAppApiUser>()
                    .set(GameTencentAppApiUser::getIsBack, backStatus.getBackStatus())
                    .eq(GameTencentAppApiUser::getId, userLog.getId())
            );
        });
        return true;
    }


    private BackStatusEnum doCallback(GameTencentAppApiUser userLog, boolean mustBack) {
        boolean isBack = mustBack ? false : gameTencentAppApiBackLogService.count(new LambdaQueryWrapper<GameTencentAppApiBackLog>()
                .eq(GameTencentAppApiBackLog::getGameId, userLog.getGameId())
                .eq(GameTencentAppApiBackLog::getUserId, userLog.getUserId())
                .eq(GameTencentAppApiBackLog::getAdAccountId, userLog.getAdAccountId())
                .eq(GameTencentAppApiBackLog::getBackLog, "回传成功")
        ) > 0;
        if (isBack) {
            return BackStatusEnum.NO;
        }
        GameTencentAppCallback callback = gameTencentAppCallbackService.getUserCallback(userLog.getGameId(), userLog.getImei(), userLog.getOaid(), userLog.getAndroidId(), userLog.getIdfa(), userLog.getCaid());
        if(callback == null) {
            return BackStatusEnum.FAILED;
        }
        String callbackUrl = URIUtil.decodeURIComponent(callback.getCallback());

        TencentAppReport reportData = TencentAppReport.builder()
                .actions(Collections.singletonList(TencentAppReport.Action.builder()
                        .action_time(DateUtil.localDateTimeToSecond(userLog.getRegisterTime()))
                        .userr_id(TencentAppReport.UserId.builder()
                                .hash_imei(StringUtils.isBlank(userLog.getImei()) ? null : Md5Util.encrypt32(userLog.getImei().toLowerCase()).toLowerCase())
                                .hash_oaid(StringUtils.isBlank(userLog.getOaid()) ? null : Md5Util.encrypt32(userLog.getOaid()).toLowerCase())
                                .hash_android_id(StringUtils.isBlank(userLog.getAndroidId()) ? null : Md5Util.encrypt32(userLog.getAndroidId()).toLowerCase())
                                .hash_idfa(StringUtils.isBlank(userLog.getIdfa()) ? null : Md5Util.encrypt32(userLog.getIdfa().toUpperCase()).toLowerCase())
                                .caid(userLog.getCaid())
                                .build())
                        .action_type("ACTIVATE_APP")
                        .trace(callback.getClickId())
                        .build()))
                .build();

        BackStatusEnum backStatus;
        String backLog;
        try {
            ResponseEntity<String> response = restTemplate.postForEntity(callbackUrl, reportData, String.class);
            if (response.getStatusCode().is2xxSuccessful()) {
                backStatus = BackStatusEnum.SUCCESS;
                backLog = "回传成功";
            } else {
                backStatus = BackStatusEnum.FAILED;
                backLog = "回传识别，失败信息：" + response.getBody();
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            backStatus = BackStatusEnum.FAILED;
            backLog = "回传识别，失败原因：" + e.getMessage();
        }

        GameTencentAppApiBackLog gameTencentBackLog = GameTencentAppApiBackLog.builder()
                .gameId(userLog.getGameId())
                .userId(userLog.getUserId())
                .adAccountId(userLog.getAdAccountId())
                .actionTime(userLog.getRegisterTime())
                .createTime(LocalDateTime.now())
                .actionType(ActionTypeEnum.REGISTER.getActionType())
                .imei(userLog.getImei())
                .oaid(userLog.getOaid())
                .androidId(userLog.getAndroidId())
                .idfa(userLog.getIdfa())
                .caid(userLog.getCaid())
                .backLog(backLog)
                .build();
        gameTencentAppApiBackLogService.save(gameTencentBackLog);
        return backStatus;
    }
}
